from test_base import TestBase


class TestMdToCodeblock(TestBase):
    def test_md_to_codeblock(self):
        # Define input and output paths
        input_file = "test/resources/example_md.md"
        output_dir = "test_output/codeblocks"

        # Run the tool using the base class method
        self.run_script("parser/cli_md_to_codeblock.py", input_file, output_dir)

        # Verify the output directory is not empty
        self.verify_output_dir(output_dir)



from collections.abc import Generator
from pathlib import Path
from tempfile import NamedTemporaryFile

from dify_plugin import Tool
from dify_plugin.entities.tool import ToolInvokeMessage

from scripts.services.svc_md_to_codeblock import convert_md_to_codeblock
from scripts.utils.file_utils import get_meta_data
from scripts.utils.logger_utils import get_logger
from scripts.utils.mimetype_utils import MimeType
from scripts.utils.param_utils import get_md_text_from_tool_params, get_param_value


class MarkdownToCodeblockTool(Tool):
    logger = get_logger(__name__)

    def _invoke(self, tool_parameters: dict) -> Generator[ToolInvokeMessage, None, None]:
        """
        invoke tools
        """

        # get parameters
        md_text = get_md_text_from_tool_params(tool_parameters)
        is_compress = get_param_value(tool_parameters, "is_compress", "true")
        compress = "true" == is_compress.lower()

        # create a temporary output path
        with NamedTemporaryFile(suffix=".zip" if compress else ".txt", delete=False) as temp_file:
            temp_output_path = Path(temp_file.name)

        try:
            # convert markdown to codeblocks using the shared function
            created_files = convert_md_to_codeblock(md_text, temp_output_path, compress=compress)

            # generate blob messages based on the created files
            if compress:
                # single ZIP file
                yield self.create_blob_message(
                    blob=created_files[0].read_bytes(),
                    meta=get_meta_data(
                        mime_type=MimeType.ZIP,
                        output_filename=tool_parameters.get("output_filename"),
                    ),
                )
            else:
                # multiple code files
                for index, file_path in enumerate(created_files):
                    # determine MIME type based on file suffix
                    suffix = file_path.suffix.lower()
                    mime_type = MimeType.TXT  # default
                    if suffix == ".css":
                        mime_type = MimeType.CSS
                    elif suffix == ".csv":
                        mime_type = MimeType.CSV
                    elif suffix == ".py":
                        mime_type = MimeType.PY
                    elif suffix == ".json":
                        mime_type = MimeType.JSON
                    elif suffix in (".js", ".javascript"):
                        mime_type = MimeType.JS
                    elif suffix in (".sh", ".bash"):
                        mime_type = MimeType.SH
                    elif suffix == ".svg":
                        mime_type = MimeType.SVG
                    elif suffix == ".xml":
                        mime_type = MimeType.XML
                    elif suffix == ".html":
                        mime_type = MimeType.HTML
                    elif suffix == ".md":
                        mime_type = MimeType.MD
                    elif suffix in (".yaml", ".yml"):
                        mime_type = MimeType.YAML
                    elif suffix == ".rb":
                        mime_type = MimeType.RUBY
                    elif suffix == ".php":
                        mime_type = MimeType.PHP
                    elif suffix == ".java":
                        mime_type = MimeType.JAVA

                    yield self.create_blob_message(
                        blob=file_path.read_bytes(),
                        meta=get_meta_data(
                            mime_type=mime_type,
                            output_filename=(tool_parameters.get("output_filename") or "code")
                            + (("_" + str(index + 1)) if len(created_files) > 1 else ""),
                        ),
                    )
        except Exception as e:
            self.logger.exception("Failed to convert markdown to codeblocks")
            raise e
        finally:
            # clean up temporary files
            if temp_output_path.exists():
                temp_output_path.unlink()
            # clean up any other created files if not compressed
            if not compress and "created_files" in locals():
                for file_path in created_files:
                    if file_path.exists():
                        file_path.unlink()



class CodeBlock:
    def __init__(self, lang_type: str, code: str):
        self.lang_type = lang_type
        self.code = code

    @property
    def code_bytes(self) -> bytes:
        return self.code.encode("utf-8")



#!/usr/bin/env python3
"""
MdToLinkedImage service
"""

import re
import zipfile
from pathlib import Path
from tempfile import NamedTemporaryFile

import httpx
import markdown
from bs4 import BeautifulSoup

from scripts.utils.logger_utils import get_logger
from scripts.utils.markdown_utils import get_md_text

logger = get_logger(__name__)

# MIME type mapping
MIME_TYPE_MAP = {
    "image/png": ".png",
    "image/jpeg": ".jpg",
    "image/jpg": ".jpg",
    "image/gif": ".gif",
    "image/webp": ".webp",
    "image/svg+xml": ".svg",
    "application/zip": ".zip",
    "image/bmp": ".bmp",
}


def get_extension_by_mime_type(mime_type: str) -> str:
    """Get file extension by MIME type"""
    return MIME_TYPE_MAP.get(mime_type.lower(), ".png")


def extract_image_urls(md_text: str) -> list[str]:
    """Extract image URLs from Markdown text"""
    html = markdown.markdown(text=md_text, extensions=["extra", "toc"])

    image_urls: list[str] = []
    try:
        soup = BeautifulSoup(html, "html.parser")
        img_tags = soup.find_all("img")
        image_urls = [img.get("src") for img in img_tags if img.get("src")]
    except Exception as e:
        logger.warning(f"Warning: Failed to extract image URLs by HTML parser, trying regex: {e}")

        # Fallback: Use regex
        markdown_image_pattern = re.compile(r"!\[.*?]\(.*?\)")
        match_image_tags = re.findall(markdown_image_pattern, md_text)
        for img in match_image_tags:
            url_match = re.findall(r"\((.*?)\)", img)
            if url_match:
                image_urls.append(url_match[0])

    # Filter invalid URLs
    result_image_urls = []
    for url in image_urls:
        if not url or not url.lower().startswith("http"):
            continue
        elif url in result_image_urls:
            continue
        else:
            result_image_urls.append(url)

    return result_image_urls


def convert_md_to_linked_image(
    md_text: str, output_path: Path, compress: bool = False, is_strip_wrapper: bool = False
) -> list[Path]:
    """
    Extract image links from Markdown and download them as files
    Args:
        md_text: Markdown text to process
        output_path: Path to save the output files
        compress: Whether to compress all images into a ZIP file
        is_strip_wrapper: Whether to remove code block wrapper if present
    Returns:
        List of paths to the created files
    Raises:
        ValueError: If input processing fails
        Exception: If conversion fails
    """
    # Process Markdown text
    processed_md = get_md_text(md_text, is_strip_wrapper=is_strip_wrapper)

    # Extract image URLs
    image_urls = extract_image_urls(processed_md)

    if not image_urls:
        raise ValueError("No image URLs found in the input text")

    # Prepare downloaded images
    downloaded_images = []
    for url in image_urls:
        try:
            response = httpx.get(url, timeout=120)
            if response.status_code != 200:
                logger.warning(
                    f"Warning: Failed to download image from URL: {url}, HTTP status code: {response.status_code}"
                )
                continue

            content_type = response.headers.get("Content-Type", "image/png")
            downloaded_images.append({"blob": response.content, "mime_type": content_type})
            logger.info(f"Downloaded: {url}")

        except Exception as e:
            logger.warning(f"Warning: Failed to download image from URL: {url}, error: {e}")
            continue

    if not downloaded_images:
        # raise Exception("No images were successfully downloaded")
        return []

    # Output files
    created_files = []
    if compress:
        # Compress to ZIP file
        try:
            with (
                NamedTemporaryFile(suffix=".zip", delete=True) as temp_zip_file,
                zipfile.ZipFile(temp_zip_file.name, mode="w", compression=zipfile.ZIP_DEFLATED) as zip_file,
            ):
                for idx, image_data in enumerate(downloaded_images, 1):
                    suffix = get_extension_by_mime_type(image_data["mime_type"])
                    with NamedTemporaryFile(delete=True) as temp_file:
                        temp_file.write(image_data["blob"])
                        temp_file.flush()
                        zip_file.write(temp_file.name, arcname=f"image_{idx}{suffix}")
                zip_file.close()

                output_path.write_bytes(Path(zip_file.filename).read_bytes())
                created_files.append(output_path)
                logger.info(f"Successfully created ZIP file with {len(downloaded_images)} images: {output_path}")

        except Exception as e:
            raise Exception(f"Failed to create ZIP file: {e}")
    else:
        # Save as separate files
        try:
            # If output path is a directory, create directory
            if output_path.suffix == "":
                output_path.mkdir(parents=True, exist_ok=True)
                base_path = output_path
            else:
                # If output path is a file, use parent directory as base path
                base_path = output_path.parent

            for index, image_data in enumerate(downloaded_images):
                suffix = get_extension_by_mime_type(image_data["mime_type"])
                if len(downloaded_images) > 1:
                    file_path = base_path / f"{output_path.stem}_{index + 1}{suffix}"
                else:
                    file_path = output_path if output_path.suffix else base_path / f"{output_path.name}{suffix}"

                file_path.parent.mkdir(parents=True, exist_ok=True)
                file_path.write_bytes(image_data["blob"])
                created_files.append(file_path)
                logger.info(f"Successfully saved image to {file_path}")

        except Exception as e:
            raise Exception(f"Failed to save images: {e}")

    return created_files



from collections.abc import Generator
from pathlib import Path
from tempfile import NamedTemporaryFile

from dify_plugin import Tool
from dify_plugin.entities.tool import ToolInvokeMessage
from dify_plugin.file.file import File

from scripts.services.svc_md_to_pptx import convert_md_to_pptx
from scripts.utils.file_utils import get_meta_data
from scripts.utils.logger_utils import get_logger
from scripts.utils.mimetype_utils import MimeType
from scripts.utils.param_utils import get_md_text_from_tool_params


class MarkdownToPptxTool(Tool):
    logger = get_logger(__name__)

    def _invoke(self, tool_parameters: dict) -> Generator[ToolInvokeMessage, None, None]:
        """
        invoke tools
        """

        # get parameters
        md_text = get_md_text_from_tool_params(tool_parameters)
        pptx_template_file: File | None = tool_parameters.get("pptx_template_file")

        # check parameters
        if pptx_template_file and not isinstance(pptx_template_file, File):
            raise ValueError("Not a valid file for pptx template file")

        temp_pptx_template_file: NamedTemporaryFile | None = None
        temp_pptx_template_file_path: Path | None = None
        try:
            if pptx_template_file:
                temp_pptx_template_file = NamedTemporaryFile(delete=False, suffix=".pptx")
                temp_pptx_template_file.write(pptx_template_file.blob)
                temp_pptx_template_file.close()
                temp_pptx_template_file_path = Path(temp_pptx_template_file.name)

            # create a temporary output PPTX file
            with NamedTemporaryFile(suffix=".pptx", delete=False) as temp_pptx_file:
                temp_pptx_output_path = Path(temp_pptx_file.name)

            # convert markdown to pptx using the shared function
            output_path = convert_md_to_pptx(md_text, temp_pptx_output_path, temp_pptx_template_file_path)

            # read the result bytes
            result_file_bytes = output_path.read_bytes()

        except Exception as e:
            self.logger.exception("Failed to convert markdown text to PPTX file")
            raise e
        finally:
            # clean up temporary files
            if temp_pptx_template_file:
                Path(temp_pptx_template_file.name).unlink(missing_ok=True)
            if "temp_pptx_output_path" in locals():
                temp_pptx_output_path.unlink(missing_ok=True)

        yield self.create_blob_message(
            blob=result_file_bytes,
            meta=get_meta_data(
                mime_type=MimeType.PPTX,
                output_filename=tool_parameters.get("output_filename"),
            ),
        )
        return



from collections.abc import Generator

import markdown
from dify_plugin import Tool
from dify_plugin.entities.tool import ToolInvokeMessage
from html2docx import html2docx

from scripts.utils.file_utils import get_meta_data
from scripts.utils.mimetype_utils import MimeType
from scripts.utils.param_utils import get_md_text_from_tool_params


class MarkdownToDocxTool(Tool):
    def _invoke(self, tool_parameters: dict) -> Generator[ToolInvokeMessage, None, None]:
        """
        invoke tools
        """
        # get parameters
        md_text = get_md_text_from_tool_params(tool_parameters, is_strip_wrapper=True)
        try:
            # Legacy: using markdowntodocx lib
            # with NamedTemporaryFile(suffix=".docx", delete=True) as temp_docx_file:
            #     markdownconverter.markdownToWordFromString(string=md_text, outfile=temp_docx_file)
            #     result_file_bytes = Path(temp_docx_file.name).read_bytes()

            html = markdown.markdown(text=md_text, extensions=["extra", "toc"])
            output_buf = html2docx(html, title="My Document")
            result_file_bytes = output_buf.getvalue()
        except Exception as e:
            self.logger.exception("Failed to convert file")
            yield self.create_text_message(f"Failed to convert markdown text to DOCX file, error: {str(e)}")
            return

        yield self.create_blob_message(
            blob=result_file_bytes,
            meta=get_meta_data(
                mime_type=MimeType.DOCX,
                output_filename=tool_parameters.get("output_filename"),
            ),
        )
        return



from test_base import TestBase


class TestMdToHtmlText(TestBase):
    def test_md_to_html_text(self):
        # Define input path
        input_file = "test/resources/example_md.md"

        # Run the tool using the base class method and capture output
        result = self.run_script_with_output("parser/cli_md_to_html_text.py", input_file)

        # Verify the output is not empty
        self.assertNotEqual(result.stdout.strip(), "", "Output is empty")



#!/usr/bin/env python3
"""
MdToPptx service
"""

import os
import subprocess
import sys
from pathlib import Path
from tempfile import NamedTemporaryFile, TemporaryDirectory

from scripts.utils.markdown_utils import get_md_text


def convert_md_to_pptx(
    md_text: str, output_path: Path, template_path: Path | None = None, is_strip_wrapper: bool = False
) -> Path:
    """
    Convert Markdown text to PPTX format using md2pptx
    Args:
        md_text: Markdown text to convert
        output_path: Path to save the output PPTX file
        template_path: Path to PPTX template file (optional)
        is_strip_wrapper: Whether to remove code block wrapper if present
    Returns:
        Path to the created PPTX file
    Raises:
        ValueError: If input processing fails or disallowed macros are found
        Exception: If conversion fails
    """
    # Process Markdown text
    processed_md = get_md_text(md_text, is_strip_wrapper=is_strip_wrapper)

    # Check for disallowed macros
    if "``` run-python" in processed_md:
        raise ValueError("The `run-python` macro of md2pptx is not allowed.")

    # Get md2pptx directory path
    script_dir = Path(__file__).resolve().parent.parent  # Go up two levels: scripts/lib -> scripts
    md2pptx_dir = script_dir / "md2pptx-6.2.1"

    if not md2pptx_dir.exists():
        raise Exception(f"md2pptx directory not found: {md2pptx_dir}. Please ensure md2pptx is properly installed.")

    # Determine template file
    final_template_path = template_path
    if not final_template_path:
        # Use default template
        default_template = script_dir.parent / "assets" / "template" / "pptx_template.pptx"
        if default_template.exists():
            final_template_path = default_template

    # Convert to PPTX
    try:
        with TemporaryDirectory() as temp_dir:
            # Prepare metadata
            metadata = f"tempDir: {temp_dir}\n"
            if final_template_path:
                metadata += f"template: {final_template_path}\n"
            metadata += "DeleteFirstSlide: yes\n"

            full_md_text = metadata + "\n" + processed_md

            # Create temporary Markdown file
            with NamedTemporaryFile(mode="w", suffix=".md", delete=False, encoding="utf-8") as md_file:
                md_file.write(full_md_text)
                md_file_path = md_file.name

            # Run md2pptx
            md2pptx_script = md2pptx_dir / "md2pptx.py"
            cmd = [sys.executable, str(md2pptx_script), md_file_path, str(output_path)]

            try:
                result = subprocess.run(cmd, timeout=60, capture_output=True, text=True)
                if result.returncode != 0:
                    raise Exception(
                        f"md2pptx failed with return code {result.returncode}."
                        f" stdout: {result.stdout},"
                        f" stderr: {result.stderr}"
                    )
            finally:
                # Delete temporary file
                if os.path.exists(md_file_path):
                    os.unlink(md_file_path)

            return output_path
    except subprocess.TimeoutExpired:
        raise Exception("md2pptx execution timed out")
    except Exception as e:
        raise Exception(f"Failed to convert to PPTX: {e}")



import os
import shutil
import subprocess
import unittest


class TestBase(unittest.TestCase):
    def setUp(self):
        # Ensure output directory exists
        os.makedirs("test_output", exist_ok=True)

        # Set up environment variables
        self.env = os.environ.copy()
        self.project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), "..", ".."))
        self.env["PYTHONPATH"] = f"{self.project_root}:{self.env.get('PYTHONPATH', '')}"

        # List to store files and directories to clean up
        self._outputs_to_clean = []

    def run_script(self, script_name, input_file, output_file):
        """Run the specified script with given input and output files/directories."""
        # Register output for cleanup
        self.register_output(output_file)

        subprocess.run(
            ["uv", "run", "python", f"scripts/{script_name}", input_file, output_file], check=True, env=self.env
        )

    def run_script_with_output(self, script_name, input_file):
        """Run the specified script with given input and capture stdout output."""
        return subprocess.run(
            ["uv", "run", "python", f"scripts/{script_name}", input_file],
            check=True,
            capture_output=True,
            text=True,
            env=self.env,
        )

    def verify_output_file(self, output_file):
        """Verify that the output file exists and is not empty."""
        self.assertTrue(os.path.exists(output_file), f"Output file {output_file} was not created")
        self.assertGreater(os.path.getsize(output_file), 0, f"Output file {output_file} is empty")

    def verify_output_dir(self, output_dir):
        """Verify that the output directory exists and is not empty."""
        self.assertTrue(os.path.exists(output_dir), f"Output directory {output_dir} was not created")
        self.assertGreater(len(os.listdir(output_dir)), 0, f"Output directory {output_dir} is empty")

    def register_output(self, output_path):
        """Register a file or directory for cleanup after the test."""
        if output_path not in self._outputs_to_clean:
            self._outputs_to_clean.append(output_path)

    def register_outputs(self, output_paths):
        """Register multiple files or directories for cleanup after the test."""
        for output_path in output_paths:
            self.register_output(output_path)

    def tearDown(self):
        # Clean up all registered output files and directories
        for output_path in reversed(self._outputs_to_clean):
            if os.path.exists(output_path):
                if os.path.isdir(output_path):
                    shutil.rmtree(output_path)
                else:
                    os.remove(output_path)

        # Also clean up any numbered PNG files that might be generated by some tests
        for file in os.listdir("test_output"):
            if file.startswith("test_") and file.endswith(".png"):
                file_path = os.path.join("test_output", file)
                if os.path.exists(file_path):
                    os.remove(file_path)



from collections.abc import Generator

from dify_plugin import Tool
from dify_plugin.entities.tool import ToolInvokeMessage

from scripts.utils import MarkdownUtils
from scripts.utils.file_utils import get_meta_data
from scripts.utils.logger_utils import get_logger
from scripts.utils.mimetype_utils import MimeType
from scripts.utils.param_utils import get_md_text_from_tool_params


class MarkdownToHtmlTool(Tool):
    logger = get_logger(__name__)

    def _invoke(self, tool_parameters: dict) -> Generator[ToolInvokeMessage, None, None]:
        """
        invoke tools
        """
        # get parameters
        md_text = get_md_text_from_tool_params(tool_parameters)

        try:
            html_str = MarkdownUtils.convert_markdown_to_html(md_text)
            result_file_bytes = html_str.encode("utf-8")
        except Exception as e:
            self.logger.exception("Failed to convert file")
            yield self.create_text_message(f"Failed to convert markdown text to HTML file, error: {str(e)}")
            return

        yield self.create_blob_message(
            blob=result_file_bytes,
            meta=get_meta_data(
                mime_type=MimeType.HTML,
                output_filename=tool_parameters.get("output_filename"),
            ),
        )
        return



from pathlib import Path
from tempfile import NamedTemporaryFile

from pypandoc import convert_file, convert_text


def pandoc_convert_file(md_text: str, dst_format: str, extra_args: list[str] | None) -> bytes:
    with NamedTemporaryFile(suffix=".md", delete=True) as md_file:
        md_file.write(md_text.encode("utf-8"))
        md_file.flush()

        with NamedTemporaryFile(suffix=f".{dst_format}", delete=True) as target_file:
            convert_file(
                source_file=md_file.name,
                format="markdown",
                to=dst_format,
                outputfile=target_file.name,
                extra_args=extra_args,
            )
            target_file.flush()
            return Path(target_file.name).read_bytes()


def pandoc_convert_text(md_text: str, dst_format: str) -> bytes:
    txt = convert_text(md_text, format="markdown", to=dst_format)
    return txt.encode("utf-8")



from collections.abc import Generator
from pathlib import Path
from tempfile import NamedTemporaryFile

from dify_plugin import Tool
from dify_plugin.entities.tool import ToolInvokeMessage

from scripts.services.svc_md_to_html import convert_md_to_html
from scripts.utils.file_utils import get_meta_data
from scripts.utils.logger_utils import get_logger
from scripts.utils.mimetype_utils import MimeType
from scripts.utils.param_utils import get_md_text_from_tool_params


class MarkdownToHtmlTool(Tool):
    logger = get_logger(__name__)

    def _invoke(self, tool_parameters: dict) -> Generator[ToolInvokeMessage, None, None]:
        """
        invoke tools
        """
        # get parameters
        md_text = get_md_text_from_tool_params(tool_parameters)

        try:
            # Create temporary output file
            with NamedTemporaryFile(suffix=".html", delete=False) as temp_output_file:
                temp_output_path = Path(temp_output_file.name)

            try:
                # Convert to HTML using the public service
                convert_md_to_html(md_text, temp_output_path, is_strip_wrapper=True)

                # Read the converted file content
                result_file_bytes = temp_output_path.read_bytes()

                yield self.create_blob_message(
                    blob=result_file_bytes,
                    meta=get_meta_data(
                        mime_type=MimeType.HTML,
                        output_filename=tool_parameters.get("output_filename"),
                    ),
                )
            finally:
                # Clean up temporary output file
                if temp_output_path.exists():
                    temp_output_path.unlink()
        except Exception as e:
            self.logger.exception("Failed to convert file")
            yield self.create_text_message(f"Failed to convert markdown text to HTML file, error: {str(e)}")
            raise e



#!/usr/bin/env python3
"""
Markdown to JSON converter
Converts Markdown tables to JSON or JSONL format
"""

import argparse
import sys
from pathlib import Path

# Add the scripts directory and parent directory to Python path to fix import issues
script_dir = str(Path(__file__).resolve().parent)
parent_dir = str(Path(__file__).resolve().parent.parent)

if script_dir not in sys.path:
    sys.path.insert(0, script_dir)
if parent_dir not in sys.path:
    sys.path.insert(0, parent_dir)

from scripts.services.svc_md_to_json import convert_md_to_json  # noqa: E402
from scripts.utils.logger_utils import get_logger  # noqa: E402

logger = get_logger(__name__)


def main():
    parser = argparse.ArgumentParser(
        description="Convert Markdown tables to JSON or JSONL format",
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("input", help="Input Markdown file path")
    parser.add_argument("output", help="Output JSON file path")
    parser.add_argument(
        "--style", choices=["jsonl", "json_array"], default="jsonl", help="JSON output style (default: jsonl)"
    )
    parser.add_argument("--strip-wrapper", action="store_true", help="Remove code block wrapper if present")

    args = parser.parse_args()

    # Read input
    input_path = Path(args.input)
    if not input_path.exists():
        logger.error(f"Error: Input file '{input_path}' does not exist")
        sys.exit(1)
    md_text = input_path.read_text(encoding="utf-8")

    # Convert to JSON
    output_path = Path(args.output)
    try:
        created_files = convert_md_to_json(md_text, output_path, args.style, args.strip_wrapper)
        for file_path in created_files:
            logger.info(f"Successfully converted to {file_path}")
    except Exception as e:
        logger.error(f"Error: {e}")
        sys.exit(1)


if __name__ == "__main__":
    main()



from test_base import TestBase


class TestMdToXml(TestBase):
    def test_md_to_xml(self):
        # Define input and output paths
        input_file = "test/resources/example_md.md"
        output_file = "test_output/test.xml"

        # Run the tool using the base class method
        self.run_script("parser/cli_md_to_xml.py", input_file, output_file)

        # Verify the output file is not empty
        self.verify_output_file(output_file)
